@@ -14,11 +14,9 @@
 #include "tests.h"
 #include "tmpfile-util.h"
 
-static void test_parse_etc_hosts_system(void) {
+TEST(parse_etc_hosts_system) {
         _cleanup_fclose_ FILE *f = NULL;
 
-        log_info("/* %s */", __func__);
-
         f = fopen("/etc/hosts", "re");
         if (!f) {
                 assert_se(errno == ENOENT);
@@ -37,15 +35,12 @@ static void test_parse_etc_hosts_system(void) {
         ((_addr)->family == AF_INET6 &&                                 \
          !memcmp(&(_addr)->address.in6, &(struct in6_addr) { .s6_addr = __VA_ARGS__}, 16) )
 
-static void test_parse_etc_hosts(void) {
+TEST(parse_etc_hosts) {
         _cleanup_(unlink_tempfilep) char
                 t[] = "/tmp/test-resolved-etc-hosts.XXXXXX";
 
-        log_info("/* %s */", __func__);
-
         int fd;
         _cleanup_fclose_ FILE *f;
-        const char *s;
 
         fd = mkostemp_safe(t);
         assert_se(fd >= 0);
@@ -105,14 +100,14 @@ static void test_parse_etc_hosts(void) {
         assert_se(address_equal_4(bn->addresses[2], inet_addr("1.2.3.11")));
         assert_se(address_equal_4(bn->addresses[3], inet_addr("1.2.3.12")));
 
-        assert(!hashmap_get(hosts.by_name, "within.comment"));
-        assert(!hashmap_get(hosts.by_name, "within.comment2"));
-        assert(!hashmap_get(hosts.by_name, "within.comment3"));
-        assert(!hashmap_get(hosts.by_name, "#"));
+        assert_se(!hashmap_get(hosts.by_name, "within.comment"));
+        assert_se(!hashmap_get(hosts.by_name, "within.comment2"));
+        assert_se(!hashmap_get(hosts.by_name, "within.comment3"));
+        assert_se(!hashmap_get(hosts.by_name, "#"));
 
-        assert(!hashmap_get(hosts.by_name, "short.address"));
-        assert(!hashmap_get(hosts.by_name, "long.address"));
-        assert(!hashmap_get(hosts.by_name, "multi.colon"));
+        assert_se(!hashmap_get(hosts.by_name, "short.address"));
+        assert_se(!hashmap_get(hosts.by_name, "long.address"));
+        assert_se(!hashmap_get(hosts.by_name, "multi.colon"));
         assert_se(!set_contains(hosts.no_address, "short.address"));
         assert_se(!set_contains(hosts.no_address, "long.address"));
         assert_se(!set_contains(hosts.no_address, "multi.colon"));
@@ -128,7 +123,7 @@ static void test_parse_etc_hosts(void) {
         assert_se(!set_contains(hosts.no_address, "foobar.foo.foo"));
 }
 
-static void test_parse_file(const char *fname) {
+static void test_parse_file_one(const char *fname) {
         _cleanup_(etc_hosts_free) EtcHosts hosts = {};
         _cleanup_fclose_ FILE *f;
 
@@ -138,14 +133,9 @@ static void test_parse_file(const char *fname) {
         assert_se(etc_hosts_parse(&hosts, f) == 0);
 }
 
-int main(int argc, char **argv) {
-        test_setup_logging(LOG_DEBUG);
-
-        if (argc == 1) {
-                test_parse_etc_hosts_system();
-                test_parse_etc_hosts();
-        } else
-                test_parse_file(argv[1]);
-
-        return 0;
+TEST(parse_file) {
+        for (int i = 1; i < saved_argc; i++)
+                test_parse_file_one(saved_argv[i]);
 }
+
+DEFINE_TEST_MAIN(LOG_DEBUG);
