@@ -2,6 +2,7 @@
 
 #include <pthread.h>
 
+#include "mempool.h"
 #include "process-util.h"
 #include "set.h"
 #include "tests.h"
@@ -15,6 +16,9 @@ static void* thread(void *p) {
         assert_se(*s);
 
         assert_se(!is_main_thread());
+        assert_se(mempool_enabled);
+        assert_se(!mempool_enabled());
+
         assert_se(set_size(*s) == NUM);
         *s = set_free(*s);
 
@@ -29,7 +33,10 @@ static void test_one(const char *val) {
 
         log_info("Testing with SYSTEMD_MEMPOOL=%s", val);
         assert_se(setenv("SYSTEMD_MEMPOOL", val, true) == 0);
+
         assert_se(is_main_thread());
+        assert_se(mempool_enabled);    /* It is a weak symbol, but we expect it to be available */
+        assert_se(!mempool_enabled());
 
         assert_se(s = set_new(NULL));
         for (i = 0; i < NUM; i++)
@@ -41,13 +48,11 @@ static void test_one(const char *val) {
         assert_se(!s);
 }
 
-int main(int argc, char *argv[]) {
-        test_setup_logging(LOG_DEBUG);
-
+TEST(disable_mempool) {
         test_one("0");
         /* The value $SYSTEMD_MEMPOOL= is cached. So the following
          * test should also succeed. */
         test_one("1");
-
-        return 0;
 }
+
+DEFINE_TEST_MAIN(LOG_DEBUG);
