@@ -15,8 +15,8 @@ systemd-cgtop --cpu=percentage
 systemd-cgtop --cpu=time
 systemd-cgtop -P
 systemd-cgtop -k
-# FIXME: https://github.com/systemd/systemd/issues/25248
-#systemd-cgtop --recursive=no
+systemd-cgtop --recursive=no -P
+systemd-cgtop --recursive=no -k
 systemd-cgtop --depth=0
 systemd-cgtop --depth=100
 
@@ -29,4 +29,5 @@ systemd-cgtop -p -t -c -m -i
 (! systemd-cgtop --order=foo)
 (! systemd-cgtop --depth=-1)
 (! systemd-cgtop --recursive=foo)
+(! systemd-cgtop --recursive=no)
 (! systemd-cgtop --delay=1foo)
